﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: Ouroboros
 * Dátum: 2011.07.17.
 * Idő: 8:21
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;
using System.Globalization;
using System.Threading;

namespace TetheredSun
{
	/// <summary>
	/// Description of RobustDoubleParser.
	/// </summary>
	public static class RobustParser
	{
		private static CultureInfo invariantCulture;
		private static CultureInfo localCulture;
		private static CultureInfo currentCulture;
		private static NumberFormatInfo invariantFormat;
		private static NumberFormatInfo localFormat;
		private static NumberFormatInfo currentFormat;
		private static NumberStyles numberStyle;
		private static DateTimeStyles dateTimeStyle;
		private static bool isLocalFormat;
		
		static RobustParser()
		{
			invariantCulture = CultureInfo.InvariantCulture;
			localCulture = new CultureInfo("hu-HU");
			invariantFormat = invariantCulture.NumberFormat;
			localFormat = localCulture.NumberFormat;
			currentCulture = Thread.CurrentThread.CurrentCulture;
			currentFormat = currentCulture.NumberFormat;
			numberStyle = NumberStyles.Float;
			dateTimeStyle = DateTimeStyles.AllowWhiteSpaces | DateTimeStyles.AssumeLocal;
			isLocalFormat = (currentFormat.NumberDecimalSeparator == ",");
		}
		
		public static bool TryParse(string s, out double result)
		{
			if (Double.TryParse(s, numberStyle, currentFormat, out result)) {
				return true;
			} else if (isLocalFormat) {
				if (Double.TryParse(s, numberStyle, invariantFormat, out result)) {
					currentCulture = invariantCulture;
					currentFormat = currentCulture.NumberFormat;
					isLocalFormat = false;
					return true;
				} else {
					result = Double.NaN;
					return false;
				}
			} else {
				if (Double.TryParse(s, numberStyle, localFormat, out result)) {
					currentCulture = localCulture;
					currentFormat = localCulture.NumberFormat;
					isLocalFormat = true;
					return true;
				} else {
					result = Double.NaN;
					return false;
				}
			}
		}
		
		public static bool TryParse(string s, out DateTime result)
		{
			if (DateTime.TryParse(s, currentCulture, dateTimeStyle, out result)) {
				return true;
			} else if (isLocalFormat) {
				if (DateTime.TryParse(s, invariantCulture, dateTimeStyle, out result)) {
					currentCulture = invariantCulture;
					currentFormat = currentCulture.NumberFormat;
					isLocalFormat = false;
					return true;
				} else {
					result = DateTime.MinValue;
					return false;
				}
			} else {
				if (DateTime.TryParse(s, localCulture, dateTimeStyle, out result)) {
					currentCulture = localCulture;
					currentFormat = localCulture.NumberFormat;
					isLocalFormat = true;
					return true;
				} else {
					result = DateTime.MinValue;
					return false;
				}
			}
		}
	}
}
