﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: phil
 * Dátum: 2011.06.27.
 * Idő: 15:45
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;
using System.Drawing;
using System.Drawing.Drawing2D;
//
using TetheredSun.Controls;

namespace TetheredSun.GnuPlot
{
	/// <summary>
	/// Description of PlotInfo.
	/// </summary>
	public class PlotInfo : Expression
	{
		protected string source;
		protected string title;
		protected int decimation;
		protected int xIndex;
		protected int yIndex;
		protected int errorBarIndex;
		protected int lineType;
		protected int pointType;
		protected double lineWidth;
		protected double pointSize;
		protected PlotType type;
		protected Color colour;
		protected static string baseFormatString = "\"{0}\"{1} using {2}:{3} title \"{4}\" with {5}";
		protected static string lineFormatString = "linetype {0} linewidth {1}";
		protected static string pointFormatString = "pointtype {0} pointsize {1}";
		
		public PlotInfo(string source, string title, int decimation, int xIndex, int yIndex, int errorBarIndex, int lineType, int pointType, double lineWidth, double pointSize, PlotType type,
		                Color colour) : base(null)
		{
			this.source = source;
			this.title = title;
			this.decimation = decimation;
			this.xIndex = xIndex;
			this.yIndex = yIndex;
			this.errorBarIndex = errorBarIndex;
			this.lineType = lineType;
			this.pointType = pointType;
			this.lineWidth = lineWidth;
			this.pointSize = pointSize;
			this.type = type;
			this.colour = colour;
			if ((this.lineWidth > 0.0) && (this.lineWidth < 2.0)) this.lineWidth = 2.0;
			UpdateText();
		}
		
		
		public PlotInfo(string title, int lineType, int pointType, double lineWidth, double pointSize, PlotType type, Color lineColour) :
			this("-", title, 1, 1, 2, -1, lineType, pointType, lineWidth, pointSize, type, lineColour) { }
		
		public PlotInfo() :
			this("-", "", 1, 1, 2, -1, 1, 7, 1.0, 1.0, PlotType.Lines, Color.Black) { }
		
		public PlotInfo(Curve curve) :
			this("-", curve.Label, 1, 1, 2, -1, 1, 7, curve.Line.Width, 1.0, PlotType.Lines, curve.Line.Color)
		{
			switch (curve.Line.DashStyle) {
				case DashStyle.Dash:
					this.lineType = 2;
					break;
				case DashStyle.DashDot:
					this.lineType = 5;
					break;
				case DashStyle.DashDotDot:
					this.lineType = 7;
					break;
				case DashStyle.Dot:
					this.lineType = 4;
					break;
				case DashStyle.Solid:
				default:
					this.lineType = 1;
					break;
			}
			
			if (curve.Symbol != null) {
				this.pointSize = curve.Symbol.Size / 7.0;
				this.Type = (curve.Line.Width > 0) ? PlotType.LinesPoints : PlotType.Points;
				switch (curve.Symbol.Type) {
					case SymbolType.Cross:
						this.pointType = 2;
						break;
					case SymbolType.Diamond:
						this.pointType = 74;
						break;
					case SymbolType.DownTriangle:
						this.pointType = 11;
						break;
					case SymbolType.UpTriangle:
						this.pointType = 9;
						break;
					case SymbolType.Square:
						this.pointType = 68;
						break;
					case SymbolType.Circle:
					default:
						this.pointType = 7;
						break;
				}
				// If the curve is symbol-only, let the colour be determined by the symbol:
				if (curve.Line.Width == 0.0f) {
					if (curve.Symbol.Fill is SolidBrush) {
						colour = (curve.Symbol.Fill as SolidBrush).Color;
					}
				}
			}
			
			if (curve.ErrorBar != null) {
				this.Type = PlotType.ErrorBars;
				this.errorBarIndex = 3;
			}
		}
		
		public string Source {
			get { return source; }
			set { source = value; }
		}
		
		public string Title {
			get { return title; }
			set { title = value; }
		}
		
		public int Decimation {
			get { return decimation; }
			set { decimation = value; }
		}
		
		public int XIndex {
			get { return xIndex; }
			set { xIndex = value; }
		}
		
		public int YIndex {
			get { return yIndex; }
			set { yIndex = value; }
		}
		
		public int ErrorBarIndex {
			get { return errorBarIndex; }
			set { errorBarIndex = value; }
		}
		
		
		public int LineType {
			get { return lineType; }
			set { lineType = value; }
		}
		
		public int PointType {
			get { return pointType; }
			set { pointType = value; }
		}
		
		public double LineWidth {
			get { return lineWidth; }
			set { lineWidth = value; }
		}
		
		public double PointSize {
			get { return pointSize; }
			set { pointSize = value; }
		}
		
		public override string Text {
			get {
				UpdateText();
				return text;
			}
		}
		
		public PlotType Type {
			get { return type; }
			set { type = value; }
		}
		
		public Color Colour {
			get { return colour; }
			set { colour = value; }
		}
		
		protected void UpdateText()
		{
			string decimationString = (decimation > 1) ? String.Format(" every {0}", decimation) : String.Empty;
			
			text = String.Format(baseFormatString, source, decimationString, xIndex,
			                     (errorBarIndex >= 0) ? String.Format("{0}:{1}", yIndex, errorBarIndex) : yIndex.ToString(),
			                     title, type.Text);
			
			if (type.Key != 2) {
				text += String.Format(" " + lineFormatString, lineType, lineWidth.ToString("F2").Replace(',', '.'));
			}
			
			if (type.Key != 0) {
				text += String.Format(" " + pointFormatString, pointType, pointSize.ToString("F2").Replace(',', '.'));
			}
			
			if (colour != Color.Black) {
				text += String.Format(" linecolor rgbcolor {0}", GnuPlotTextWriter.Quote(GnuPlotTextWriter.GetColourString(colour)));
			}
		}
		
	}
}
