﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: Ouroboros
 * Dátum: 2010.10.29.
 * Idő: 11:25
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;
using System.Collections.Generic;

namespace TetheredSun.Mathematics
{
	/// <summary>
	/// Description of DoubleListExtensions.
	/// </summary>
	public static class DoubleListExtensions
	{
		private static int notNaNCount;	// The number of elements in a list that are not NaN.
 		public static bool SkipNaN = true;
 		
 		public static double[] Add(this IList<double> list, IList<double> other)
 		{
 			int length = Math.Min(list.Count, other.Count);
 			double[] result = new double[length];
 			
 			for (int i = 0; i < result.Length; i++) {
 				result[i] = list[i] + other[i];
 			}
 			
 			return result;
 		}
 		
 		public static double[] Decimate(this IList<double> list, int decimation)
 		{
 			double[] decimated = new double[list.Count / decimation];
 			
 			for (int i = 0; i < decimated.Length; i++) {
 				decimated[i] = list[i * decimation];
 			}
 			
 			return decimated;
 		}
 		
 		public static void GetExtrema(this IList<double> list, out double min, out double max)
 		{
 			double x;
 			min = Double.MaxValue;
 			max = Double.MinValue;
 			
 			for (int i = 0; i < list.Count; i++) {
 				x = list[i];
 				if (x < min) min = x;
 				if (x > max) max = x;
 			}
 		}
 		
 		public static double GetMean(this IList<double> list)
 		{
 			if (list == null) throw new NullReferenceException("Mean cannot be calculated:" +
 				                                                        " the list is a null reference.");
 			
 			if (list.Count <= 0) throw new InvalidOperationException("Mean cannot be calculated:" +
 				                                                              " the list is empty.");
 			int n = 0;
 			double sum = 0;
 			
 			foreach (double element in list) {
 				// Check if we should simply skip NaN values instead of letting them NaN out the whole mean.
 				if (!SkipNaN || !Double.IsNaN(element)) {
 					sum += element;
 					n++;
 				}
 			}
 			notNaNCount = n;
 			if (n == 0) {
 				return Double.NaN;
 			} else {
 				return (sum / (double) n);
 			}
 		}
 		
 		public static double[] GetRange(this IList<double> list, int startIndex, int count)
 		{
 			double[] range = new double[count];
 			
 			for (int i = 0; i < count; i++) {
 				range[i] = list[startIndex + i];
 			}
 			
 			return range;
 		}
 		
 		// Validated by comparison to LabVIEW 7.1 results.
 		public static double GetRms(this IList<double> list)
 		{
 			if (list == null) throw new NullReferenceException("Root mean square cannot be calculated:" +
 				                                                        " the list is a null reference.");
 			
 			if (list.Count <= 0) throw new InvalidOperationException("Root mean square cannot be calculated:" +
 				                                                              " the list is empty.");
 			
 			double sum = 0;
 			int n = 0;
 			
 			foreach (double d in list) {
 				if (!SkipNaN || !Double.IsNaN(d)) {
 					sum += d * d;
 					n++;
 				}
 			}
 			
 			if (n == 0) {
 				return Double.NaN;
 			} else {
 				return Math.Sqrt(sum / (double)n);
 			}
 		}
 		
 		// Validated by comparison to LabVIEW 7.1 results.
 		public static double GetSD(this IList<double> list, bool isSample)
 		{
 			if (list == null) throw new NullReferenceException("Standard deviation cannot be calculated:" +
 				                                                        " the list is a null reference.");
 			
 			if (list.Count <= 0) throw new InvalidOperationException("Standard deviation cannot be calculated:" +
 				                                                              " the list is empty.");
 			
 			double distance;
 			double mean = GetMean(list);
 			double sum = 0;
 			int n = 0;
 			int w;
 			
 			foreach (double d in list) {
 				if (!SkipNaN || !Double.IsNaN(d)) {
 					distance = d - mean;
 					sum += distance * distance;
 					n++;
 				}
 			}
 			
 			notNaNCount = n;
 			w = isSample ? (n - 1) : n; 
 			if (w == 0) {
 				return Double.NaN;
 			} else {
 				return Math.Sqrt( sum / (double)w);
 			}
 		}
 		
 		// Standard error of the mean based on sample standard deviation.
 		public static double GetSE(this IList<double> list)
 		{
 			if (list == null)
 				throw new NullReferenceException("Cannot obtain standard error: the list is a null reference.");
 			if (list.Count <= 0)
 				throw new InvalidOperationException("Cannot obtain standard error: the list is empty.");
 			double sd = list.GetSD(true);
 			int n = SkipNaN ? notNaNCount : list.Count;
 			if (n == 0) {
 				return Double.NaN;
 			} else {
 				return (sd / Math.Sqrt(n));
 			}	
 		}
 		
 		public static double GetSum(this IList<double> list)
 		{
 			double sum = 0.0;
 			
 			for (int i = 0; i < list.Count; i++) {
 				sum += list[i];
 			}
 			
 			return sum;
 		}
 		
 		public static double[] MultiplyBy(this IList<double> list, double multiplier)
 		{
 			double[] result = new double[list.Count];
 			for (int i = 0; i < list.Count; i++) {
 				result[i] = list[i] * multiplier;
 			}
 			
 			return result;
 		}
 		
 		public static double[] Normalise(this IList<double> list)
 		{
 			double max = Double.MinValue;
 			double min = Double.MaxValue;
 			double normalisingFactor;
 			double[] normalised = new double[list.Count];
 			
 			for (int i = 0; i < list.Count; i++) {
 				if (list[i] < min) {
 					min = list[i];
 				} else if (list[i] > max) {
 					max = list[i];
 				}
 			}
 			
 			normalisingFactor = 1.0 / (max - min);
 			
 			for (int i = 0; i < list.Count; i++) {
 				normalised[i] = (list[i] - min) * normalisingFactor;
 			}
 			
 			return normalised;
 		}
 		
 		// Tested on the sample at http://en.wikipedia.org/wiki/Quantile
 		public static double Quantile(this IList<double> list, double  parameter)
 		{
 			if (list == null)
 				throw new NullReferenceException("Quantile cannot be calculated: 'list' is a null reference.");
 			int n = list.Count;
 			if (n <= 0)
 				throw new ArgumentException("Quantile cannot be calculated: 'list' is an empty list.");
 			if ((parameter <= 0.0) || (parameter >= 1.0))
 				throw new ArgumentException("Quantile parameter should be between 0 and 1 (exclusive)," +
 				                            " whereas the current value is " + parameter.ToString() + ".");
 			
 			double quantile = 0.0;
 			double fractionalIndex = Math.Round(parameter * n, MidpointRounding.AwayFromZero);
 			bool isInteger = (fractionalIndex == parameter * n);
 			int index = (int)fractionalIndex - 1;
 			List<double> values = new List<double>(list);
 			
 			if (index < 0) index = 0;
 			values.Sort();
 			if (isInteger) {
 				quantile = (values[index] + values[index + 1]) / 2.0;
 			} else {
 				quantile = values[index];
 			}
 						 			
 			return quantile;
 		}
	
 		public static double[] Subtract(this IList<double> list, IList<double> other)
 		{
 			int length = Math.Min(list.Count, other.Count);
 			double[] result = new double[length];
 			
 			for (int i = 0; i < result.Length; i++) {
 				result[i] = list[i] - other[i];
 			}
 			
 			return result;
 		}
 		
		public static string ToString(this IList<double> list, string format)
		{
			return ToString(list, format, "\t");
		}
		
		public static string ToString(this IList<double> list, string format, string separator)
		{
			string text = String.Empty;
			
			if (list.Count < 1) return text;
			
			text += list[0].ToString(format);
			
			for (int i = 1; i < list.Count; i++) {
				text += separator + list[i].ToString(format);
			}
			
			return text;
		}
	}
}
