﻿/*
 * Created by SharpDevelop.
 * User: phil
 * Date: 2010.08.17.
 * Time: 11:55
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;

namespace TetheredSun.Mathematics
{
	/// <summary>
	/// Description of Histogram.
	/// </summary>
	public class Histogram
 	{
 		protected double min;
 		protected double max;
 		protected double binWidth;
 		protected List<Bin> bins;
 		protected int count = 0;
 		
 		public Histogram() { }
 		
 		public void Add(IList<double> data)
 		{
 			double dataMin;
 			double dataMax;
 			if (data.Count <= 0) return;
 			data.GetExtrema(out dataMin, out dataMax);
 			
 			if (bins == null) {
 				double currentMax;
 				min = dataMin;
 				binWidth = 3.5 * data.GetSD(true) / Math.Pow(data.Count, 1.0 / 3.0);
 				if (min >= dataMax) return;
 				bins = new List<Bin>();
				currentMax = min;
				while (currentMax < dataMax) {
					bins.Add(new Bin(currentMax, binWidth));
					currentMax += binWidth;
				}
 				max = currentMax;
 			} else {
 				if (dataMin < min) ExtendDown(dataMin);
 				if (dataMax > max) ExtendUp(dataMax);
 			}
 			int index;
 			foreach (double d in data) {
 				index = GetIndex(d);
 				if (index >= 0) {
 					count++;
 					bins[index]++;
 				}
 			}
 		}
 		
 		public Point<double>[] GetFrequencies(bool isRelative)
 		{
 			if (bins == null) return null;
 			Point<double>[] result = new Point<double>[bins.Count];
 			
 			for (int i = 0; i < bins.Count; i++) {
 				result[i] = new Point<double>(
 					bins[i].Min,
 					isRelative ? (double)bins[i].Count / (this.count * binWidth) : bins[i].Count
 				);
 			}
 			
 			return result;
 		}
 		
 		public void Reset()
 		{
 			bins = null;
 			count = 0;
 		}
 		
 		protected void ExtendDown(double valueToInclude)
 		{
 			if (binWidth <= 0.0) return;
 			while (valueToInclude < min) {
 				bins.Insert(0, new Bin(min - binWidth));
 				min -= binWidth;
 			}
 		}
 		
 		protected void ExtendUp(double valueToInclude)
 		{
 			if (binWidth <= 0.0) return;
 			while (valueToInclude > max) {
 				bins.Add(new Bin(max, binWidth));
 				max += binWidth;				
 			}
 		}
 		
 		protected int GetIndex(double d)
 		{
 			if (d < min) return -1;
 			if (d > max) return -2;
 			if (binWidth == 0.0) return -3;
 			if (Double.IsNaN(d)) return -4;
 			return (int)Math.Floor((d - min) / binWidth);
 		}
 	}
}
